@@ -3,7 +3,10 @@
 
 from random import shuffle
 
-import pickle
+try:
+	import cPickle as pickle
+except ImportError:
+	import pickle
 
 import os, sys, re
 
@@ -11,7 +14,7 @@ from grab.error import GrabError
 
 from time import time, sleep
 
-from threading import Thread
+import multiprocessing
 
 from mako.lookup import TemplateLookup
 
@@ -132,17 +135,13 @@ def simple_plural(n, s):
 	else:
 		return s + "s"
 
-class State(object):
-	def __init__(self):
-		self.progress = 0.0
-
-def show_progress(s, state):
-	progress = round(state.progress, 2)
+def show_progress(s, shared_object):
+	progress = round(shared_object.value, 2)
 	return "\033[0;32m[{}%]\033[0m ".format(progress)+s
 
-def parse_site(queue, articles, state):
-	while queue:
-		site = queue.pop()
+def parse_site(queue, articles, progress):
+	while not queue.empty():
+		site = queue.get()
 		s = "Got {} {} from {}"
 		
 		if site not in config.sites_to_parse:
@@ -150,25 +149,28 @@ def parse_site(queue, articles, state):
 		else:
 			d = config.sites_to_parse[site]
 		
-		state.progress += 100.0 / (len(config.sites_to_parse) + len(config.rss_feeds)) / 2.0
-		log(show_progress("Parsing articles from {}".format(site), state))
+		update_progress(progress,
+			100.0 / (len(config.sites_to_parse) + len(config.rss_feeds)) / 2.0)
+		log(show_progress("Parsing articles from {}".format(site), progress))
 		
 		if 'module' not in d:
-			url = d['url']
-			short_name = d['short-name']
-			icon = d['icon']
-			color = d['color']
+			url = d.get('url', 'about:blank')
+			short_name = d.get('short-name', 'unknown')
+			icon = d.get('icon', 'about:blank')
+			color = d.get('color', '#000')
 			if not len(color):
-				color = '#FFF'
+				color = '#000'
 			if not len(icon):
 				icon = 'about:blank'
 			
 			try:
 				new_articles = parser.parse_rss(url, short_name, icon, color)
-				articles += new_articles
-				state.progress += 100.0 / (len(config.sites_to_parse) + len(config.rss_feeds)) / 2
+				for i in new_articles:
+					articles.put(i)
+				update_progress(progress,
+					100.0 / (len(config.sites_to_parse) + len(config.rss_feeds)) / 2.0)
 				log(show_progress(s.format(len(new_articles),
-					simple_plural(len(new_articles), 'article'), site), state))
+					simple_plural(len(new_articles), 'article'), site), progress))
 			except Exception as error:
 				log('Fail')
 				log(str(error), f=sys.stderr)
@@ -178,38 +180,54 @@ def parse_site(queue, articles, state):
 			
 			try:
 				found = module.get_articles(**kwargs)
-				articles += found
-				state.progress += 100.0 / (len(config.sites_to_parse) + len(config.rss_feeds)) / 2.0
+				for i in found:
+					articles.put(i)
+				update_progress(progress,
+					100.0 / (len(config.sites_to_parse) + len(config.rss_feeds)) / 2.0)
 				log(show_progress(s.format(len(found),
-					simple_plural(len(found), 'article'), site), state))
+					simple_plural(len(found), 'article'), site), progress))
 			except Exception as error:
 				log("Failed to parse articles from {}".format(site))
 				log(str(error), f=sys.stderr)
 
-def dump_articles():
-	"""Dump articles to ~/.tech-parser/articles_dumped"""
+def update_progress(shared_object, num):
+	shared_object.value += num
+
+def dump_articles(filename="articles_dumped"):
+	"""Dump articles to ~/.tech-parser/<filename>"""
 	
-	articles = []
+	m = multiprocessing.Manager()
 	
-	state = State()
+	articles = m.Queue()
 	
-	main_queue = [i for i in config.sites_to_parse]
-	main_queue += [i for i in config.rss_feeds]
+	progress = m.Value('d', 0.0, lock=False)
+	
+	main_queue = m.Queue()
+	
+	for i in config.sites_to_parse:
+		main_queue.put(i)
+
+	for i in config.rss_feeds:
+		main_queue.put(i)
 	
 	s = "\033[0;32m[{}%]\033[0m Parsing articles from {}... "
 	
-	threads = []
+	pool = multiprocessing.Pool(processes=config.num_threads)
 	
 	for i in range(config.num_threads):
-		threads.append(Thread(target=parse_site, args=(main_queue,articles,state)))
+		pool.apply_async(parse_site, (main_queue, articles, progress))
 	
-	for thread in threads:
-		thread.start()
+	pool.close()
+	pool.join()
 	
-	for thread in threads:
-		thread.join()
+	articles_before = [i[0] for i in load_articles()]
+	list_articles = []
+	while not articles.empty():
+		list_articles.append(articles.get())
 	
-	log("Total articles: %d" %(len(articles)))
+	log("Total articles: %d" %(len(list_articles)))
+	log("New articles: %d"
+		%(len([i for i in list_articles if i not in articles_before])))
 	
 	if config.save_articles:
 		log("Saving articles to archive...")
@@ -218,7 +236,7 @@ def dump_articles():
 		con = sqlite3.connect(os.path.join(logdir, "archive.db"))
 		cur = con.cursor()
 		
-		for article in articles:
+		for article in list_articles:
 			title = article["title"]
 			link = article["link"]
 			source = article["source"]
@@ -232,23 +250,23 @@ def dump_articles():
 		con.commit()
 		con.close()
 	
-	log("Shuffling articles...")
-	shuffle(articles)
-	
-	log("Ranking articles...")
 	num = len(recommend.get_interesting_articles(db=config.db))
 	num += len(recommend.get_blacklist(db=config.db))
+	
 	if num >= 20:
-		articles = recommend.find_similiar(articles, db=config.db)
-		articles.sort(key=lambda x: x[1], reverse=True)
+		log("Ranking articles...")
+		list_articles = recommend.find_similiar(list_articles, db=config.db)
+		list_articles.sort(key=lambda x: x[1], reverse=True)
 	else:
-		articles = [[a, -1] for a in articles]
+		log("Shuffling articles...")
+		shuffle(list_articles)
+		list_articles = [[a, -1] for a in list_articles]
 	
-	log("Dumping data to file: articles_dumped...")
+	log("Dumping data to file: {}...".format(filename))
 	
-	dumped = pickle.dumps(articles)
+	dumped = pickle.dumps(list_articles)
 	path = os.path.join(os.path.expanduser("~"), ".tech-parser")
-	path = os.path.join(path, "articles_dumped")
+	path = os.path.join(path, filename)
 	f = open(path, "wb")
 	f.write(dumped)
 	f.close()
@@ -256,7 +274,7 @@ def dump_articles():
 	log("Done!")
 
 def dump_articles_per(s):
-	"""Dump articles per S seconds"""
+	"""Dump articles per <s> seconds"""
 	
 	while True:
 		if int(time()) % s == 0:
@@ -299,11 +317,16 @@ def filter_articles(articles):
 	
 	return articles_filtered
 
-def load_articles():
-	"""Load articles from ~/.tech-parser/articles_dumped"""
+def load_articles(filename="articles_dumped"):
+	"""Load articles from ~/.tech-parser/<filename>"""
+	
+	log("Reading articles from file: {}...".format(filename))
+	try:
+		f = open(os.path.join(logdir, filename), 'rb')
+	except IOError:
+		log("File '{}' doesn't exist: returning empty list".format(filename))
+		return []
 	
-	log("Reading articles from file: articles_dumped...")
-	f = open(os.path.join(logdir, "articles_dumped"), 'rb')
 	dumped = f.read()
 	f.close()
 	
@@ -501,10 +524,13 @@ class ParserDaemon(Daemon):
 		super(ParserDaemon, self).__init__(pidfile, stdout=so, stderr=se)
 	
 	def onStart(self):
-		t1 = Thread(target=dump_articles_per, args=(config.update_interval,))
-		t1.daemon = True
-		t1.start()
-		run(host=config.host, port=config.port, server=config.server)
+		p1 = multiprocessing.Process(target=dump_articles_per, args=(config.update_interval,))
+		p2 = multiprocessing.Process(target=run,
+			kwargs={'host': config.host, 'port': config.port, 'server': config.server})
+		p1.start()
+		p2.start()
+		p1.join()
+		p2.join()
 
 def is_hostname(hostname):
 	if len(hostname) > 255:
@@ -515,10 +541,13 @@ def is_hostname(hostname):
 	return all(allowed.match(x) for x in hostname.split("."))
 
 def run_server(host, port):
-	t1 = Thread(target=dump_articles_per, args=(config.update_interval,))
-	t1.daemon = True
-	t1.start()
-	run(host=host, port=port, server=config.server)
+	p1 = multiprocessing.Process(target=dump_articles_per, args=(config.update_interval,))
+	p2 = multiprocessing.Process(target=run,
+		kwargs={'host': config.host, 'port': config.port, 'server': config.server})
+	p1.start()
+	p2.start()
+	p1.join()
+	p2.join()
 
 if __name__ == "__main__":
 	parser_daemon = ParserDaemon()
